@@ -4,6 +4,7 @@
  * Copyright (C) 2001 Havoc Pennington
  * Copyright (C) 2003 Kim Woelders
  * Copyright (C) 2003 Red Hat, Inc.
+ * Includes urgency notification based on Matthew Eaton's patch for libwnck
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -104,6 +105,8 @@ struct _NetkTask
     guint was_active:1;         /* used to fixup activation behavior */
 
     guint button_activate;
+
+    guint blink_timeout_id;
 };
 
 struct _NetkTaskClass
@@ -150,6 +153,8 @@ struct _NetkTasklistPrivate
     NetkLoadIconFunction icon_loader;
     void *icon_loader_data;
     GDestroyNotify free_icon_loader_data;
+
+    gboolean show_label;
 };
 
 
@@ -162,6 +167,7 @@ static NetkTask *netk_task_new_from_window (NetkTasklist * tasklist,
 static NetkTask *netk_task_new_from_class_group (NetkTasklist * tasklist,
                                                  NetkClassGroup * class_group);
 
+static gboolean  urgent_timeout_callback(gpointer data);
 static char *netk_task_get_text (NetkTask * task);
 static GdkPixbuf *netk_task_get_icon (NetkTask * task);
 static gint netk_task_compare (gconstpointer a, gconstpointer b);
@@ -206,7 +212,9 @@ static void netk_tasklist_connect_window (NetkTasklist * tasklist,
 
 static void netk_tasklist_change_active_task (NetkTasklist * tasklist,
                                               NetkTask * active_task);
+#if 0  /* unused? */
 static gboolean netk_tasklist_change_active_timeout (gpointer data);
+#endif
 static void netk_tasklist_activate_task_window (NetkTask * task);
 
 static void netk_tasklist_update_icon_geometries (NetkTasklist * tasklist);
@@ -260,9 +268,12 @@ netk_task_class_init (NetkTaskClass * klass)
 
     object_class->finalize = netk_task_finalize;
 
-    gtk_rc_parse_string ("\n" "   style \"tasklist-button-style\"\n" "   {\n"
+    gtk_rc_parse_string ("\n" 
+                         "   style \"tasklist-button-style\"\n" 
+                         "   {\n"
                          "      GtkWidget::focus-line-width=0\n"
-                         "      GtkWidget::focus-padding=0\n" "   }\n" "\n"
+                         "      GtkWidget::focus-padding=0\n" 
+                         "   }\n" "\n"
                          "    widget \"*.tasklist-button\" style \"tasklist-button-style\"\n"
                          "\n");
 }
@@ -277,6 +288,12 @@ netk_task_finalize (GObject * object)
     if (task->tasklist->priv->active_task == task)
         netk_tasklist_change_active_task (task->tasklist, NULL);
 
+    if (task->blink_timeout_id)
+    {
+        g_source_remove (task->blink_timeout_id);
+        task->blink_timeout_id = 0;
+    }
+
     if (task->button)
     {
         gtk_widget_destroy (task->button);
@@ -473,6 +490,20 @@ netk_tasklist_finalize (GObject * object)
     G_OBJECT_CLASS (tasklist_parent_class)->finalize (object);
 }
 
+void
+netk_tasklist_set_show_label (NetkTasklist * tasklist,
+                              gboolean show_label)
+{ 
+    g_return_if_fail (NETK_IS_TASKLIST (tasklist));
+
+    if (tasklist->priv->show_label == show_label)
+        return;
+
+    tasklist->priv->show_label = show_label;
+    gtk_widget_queue_resize (GTK_WIDGET (tasklist));
+    netk_tasklist_update_lists (tasklist);
+}
+
 void
 netk_tasklist_set_grouping (NetkTasklist * tasklist,
                             NetkTasklistGroupingType grouping)
@@ -977,7 +1008,7 @@ netk_tasklist_size_allocate (GtkWidget * widget, GtkAllocation * allocation)
     i = 0;
     total_width = tasklist->priv->max_button_width * n_cols;
     total_width = MIN (total_width, allocation->width);
-/*    total_width = allocation->width;*/
+    /* total_width = allocation->width; */
     while (l != NULL)
     {
         NetkTask *task = NETK_TASK (l->data);
@@ -1198,6 +1229,8 @@ netk_tasklist_new (NetkScreen * screen)
 
     netk_tasklist_set_screen (tasklist, screen);
 
+    tasklist->priv->show_label=TRUE;
+
     return GTK_WIDGET (tasklist);
 }
 
@@ -1216,6 +1249,13 @@ netk_tasklist_free_tasks (NetkTasklist * tasklist)
         {
             NetkTask *task = NETK_TASK (l->data);
             l = l->next;
+            
+            if (task->blink_timeout_id)
+            {
+                g_source_remove (task->blink_timeout_id);
+                task->blink_timeout_id = 0;
+            }
+
             /* if we just unref the task it means we lose our ref to the
              * task before we unparent the button, which breaks stuff.
              */
@@ -1232,6 +1272,13 @@ netk_tasklist_free_tasks (NetkTasklist * tasklist)
         {
             NetkTask *task = NETK_TASK (l->data);
             l = l->next;
+
+            if (task->blink_timeout_id)
+            {
+                g_source_remove (task->blink_timeout_id);
+                task->blink_timeout_id = 0;
+            }
+
             /* if we just unref the task it means we lose our ref to the
              * task before we unparent the button, which breaks stuff.
              */
@@ -1602,6 +1649,7 @@ netk_task_position_menu (GtkMenu * menu, gint * x, gint * y,
     *push_in = TRUE;
 }
 
+#if 0 /* unused? */
 static gboolean
 netk_tasklist_change_active_timeout (gpointer data)
 {
@@ -1613,6 +1661,7 @@ netk_tasklist_change_active_timeout (gpointer data)
 
     return FALSE;
 }
+#endif
 
 static void
 netk_task_menu_activated (GtkMenuItem * menu_item, gpointer data)
@@ -1668,14 +1717,11 @@ netk_tasklist_activate_task_window (NetkTask * task)
         }
     }
 
-
     if (tasklist->priv->activate_timeout_id)
+    {
         g_source_remove (tasklist->priv->activate_timeout_id);
-
-    tasklist->priv->activate_timeout_id =
-        g_timeout_add_full (0, 500, &netk_tasklist_change_active_timeout,
-                            tasklist, NULL);
-
+    }
+    
     netk_tasklist_change_active_task (tasklist, task);
 }
 
@@ -1933,7 +1979,8 @@ netk_task_update_visible_state (NetkTask * task)
     text = netk_task_get_text (task);
     if (text != NULL)
     {
-        gtk_label_set_text (GTK_LABEL (task->label), text);
+        if (task->tasklist->priv->show_label)
+            gtk_label_set_text (GTK_LABEL (task->label), text);
         gtk_tooltips_set_tip (task->tasklist->priv->tooltips, task->button,
                               text, NULL);
         g_free (text);
@@ -1974,7 +2021,42 @@ netk_task_state_changed (NetkWindow * window, NetkWindowState changed_mask,
                 netk_task_update_visible_state (class_group_task);
         }
     }
+    
+    if (changed_mask & NETK_WINDOW_STATE_URGENT)
+    {
+        NetkTask *win_task;
 
+        win_task = g_hash_table_lookup (tasklist->priv->win_hash, window);
+        if (win_task)
+        {
+            if (win_task->blink_timeout_id)
+            {
+                g_source_remove (win_task->blink_timeout_id);
+            }
+
+            if(new_state & NETK_WINDOW_STATE_URGENT)
+            {
+                win_task->blink_timeout_id = 
+                                       g_timeout_add (500, 
+                                                      urgent_timeout_callback, 
+                                                      win_task);
+            }
+            else 
+            {
+                GtkRcStyle *style; 
+
+                style = gtk_widget_get_modifier_style (win_task->button);
+                style->color_flags[GTK_STATE_NORMAL] = 0;
+                gtk_widget_modify_style (win_task->button, style);
+                if(win_task->tasklist->priv->show_label)
+                {
+                    style = gtk_widget_get_modifier_style (win_task->label);
+                    style->color_flags[GTK_STATE_NORMAL] = 0;
+                    gtk_widget_modify_style (win_task->label, style);
+                }
+            }
+        }
+    }
 }
 
 static void
@@ -2147,13 +2229,16 @@ netk_task_create_widgets (NetkTask * task)
     gtk_widget_show (task->image);
 
     text = netk_task_get_text (task);
-    task->label = gtk_label_new (text);
-    gtk_widget_show (task->label);
 
     gtk_table_attach (GTK_TABLE (table),
                       task->image, 0, 1, 0, 1, 0, GTK_EXPAND, 0, 0);
-    gtk_table_attach (GTK_TABLE (table),
+
+    if (task->tasklist->priv->show_label) {
+        task->label = gtk_label_new (text);
+        gtk_widget_show (task->label);
+        gtk_table_attach (GTK_TABLE (table),
                       task->label, 1, 2, 0, 1, GTK_EXPAND, GTK_EXPAND, 0, 0);
+    }
 
     gtk_container_add (GTK_CONTAINER (task->button), table);
     gtk_widget_show (table);
@@ -2192,7 +2277,7 @@ netk_task_create_widgets (NetkTask * task)
             task->class_icon_changed_tag =
                 g_signal_connect (G_OBJECT (task->class_group),
                                   "icon_changed",
-				  G_CALLBACK (netk_task_class_icon_changed),
+                                  G_CALLBACK (netk_task_class_icon_changed),
                                   task);
             break;
         case NETK_TASK_WINDOW:
@@ -2212,6 +2297,59 @@ netk_task_create_widgets (NetkTask * task)
     }
 }
 
+static gboolean
+urgent_timeout_callback (gpointer data)
+{
+    NetkTask *task= (NetkTask *) data; 
+    GtkStyle *style;
+    GtkRcStyle *mod;
+    GdkColor c;
+    NetkWindowState state;
+
+    g_return_val_if_fail (task, FALSE);
+
+    state = netk_window_get_state (task->window);
+
+    if(state & NETK_WINDOW_STATE_URGENT)
+    {
+        style = gtk_widget_get_style (task->button);
+        mod = gtk_widget_get_modifier_style (task->button);
+        c = style->bg[GTK_STATE_SELECTED];
+        if(mod->color_flags[GTK_STATE_NORMAL] & GTK_RC_BG)
+        {
+            mod->color_flags[GTK_STATE_NORMAL] &= ~(GTK_RC_BG);
+            gtk_widget_modify_style(task->button, mod);
+        }
+        else
+        {
+            mod->color_flags[GTK_STATE_NORMAL] |= GTK_RC_BG;
+            mod->bg[GTK_STATE_NORMAL] = c;
+            gtk_widget_modify_style(task->button, mod);
+        }
+
+        if(task->tasklist->priv->show_label)
+        {
+            style = gtk_widget_get_style (task->label);
+            mod = gtk_widget_get_modifier_style (task->label);
+            c = style->fg[GTK_STATE_SELECTED];
+            
+            if(mod->color_flags[GTK_STATE_NORMAL] & GTK_RC_FG)
+            {
+                mod->color_flags[GTK_STATE_NORMAL] &= ~(GTK_RC_FG);
+                gtk_widget_modify_style(task->label, mod);
+            }
+            else
+            {
+                mod->color_flags[GTK_STATE_NORMAL] |= GTK_RC_FG;
+                mod->fg[GTK_STATE_NORMAL] = c;
+                gtk_widget_modify_style(task->label, mod);
+            }
+        }
+    }
+
+    return (state & NETK_WINDOW_STATE_URGENT);
+}
+ 
 static void
 draw_dot (GdkWindow * window, GdkGC * lgc, GdkGC * dgc, int x, int y)
 {
@@ -2320,8 +2458,14 @@ netk_task_new_from_window (NetkTasklist * tasklist, NetkWindow * window)
     task->window = g_object_ref (window);
     task->class_group = g_object_ref (netk_window_get_class_group (window));
     task->tasklist = tasklist;
+    task->blink_timeout_id = 0;
 
     netk_task_create_widgets (task);
+    
+    if (netk_window_get_state (window) & NETK_WINDOW_STATE_URGENT)
+    {
+        task->blink_timeout_id = g_timeout_add (500, urgent_timeout_callback, task);
+    }
 
     return task;
 }
