@@ -4,6 +4,7 @@
  * Copyright (C) 2001 Havoc Pennington
  * Copyright (C) 2003 Kim Woelders
  * Copyright (C) 2003 Red Hat, Inc.
+ * Includes urgency notification based on Matthew Eaton's patch for libwnck
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -33,6 +34,7 @@
 #endif
 
 #include "netk-window.h"
+#include "netk-class-group.h"
 #include "netk-xutils.h"
 #include "netk-private.h"
 #include "netk-enum-types.h"
@@ -55,19 +57,24 @@ static GHashTable *window_hash = NULL;
     ((window)->priv->skip_taskbar        << 5) |        \
     ((window)->priv->is_sticky           << 6) |        \
     ((window)->priv->is_hidden           << 7) |        \
-    ((window)->priv->is_fullscreen       << 8) )
+    ((window)->priv->is_fullscreen       << 8) |        \
+    ((window)->priv->is_urgent           << 9) )
 
 struct _NetkWindowPrivate
 {
     Window xwindow;
     NetkScreen *screen;
     NetkApplication *app;
+    NetkClassGroup *class_group;
     Window group_leader;
     Window transient_for;
     char *name;
     char *icon_name;
     char *session_id;
     char *session_id_utf8;
+    char *res_class;
+    char *res_name;
+
     int pid;
     int workspace;
 
@@ -97,6 +104,7 @@ struct _NetkWindowPrivate
     guint is_sticky:1;
     guint is_hidden:1;
     guint is_fullscreen:1;
+    guint is_urgent : 1;
 
     /* _NET_WM_STATE_HIDDEN doesn't map directly into an
      * externally-visible state (it determines the WM_STATE
@@ -120,6 +128,8 @@ struct _NetkWindowPrivate
     guint need_update_actions:1;
     guint need_update_wintype:1;
     guint need_update_transient_for:1;
+    guint need_update_wmclass : 1;
+    guint need_update_wmhints : 1;
 };
 
 enum
@@ -156,6 +166,7 @@ static void update_workspace (NetkWindow * window);
 static void update_actions (NetkWindow * window);
 static void update_wintype (NetkWindow * window);
 static void update_transient_for (NetkWindow * window);
+static void update_wmclass (NetkWindow *window);
 static void unqueue_update (NetkWindow * window);
 static void queue_update (NetkWindow * window);
 static void force_update_now (NetkWindow * window);
@@ -276,6 +287,8 @@ netk_window_finalize (GObject * object)
     g_free (window->priv->name);
     g_free (window->priv->icon_name);
     g_free (window->priv->session_id);
+    g_free (window->priv->res_class);
+    g_free (window->priv->res_name);
 
     if (window->priv->app)
         g_object_unref (G_OBJECT (window->priv->app));
@@ -364,6 +377,8 @@ p_netk_window_create (Window xwindow, NetkScreen * screen)
     window->priv->need_update_actions = TRUE;
     window->priv->need_update_wintype = TRUE;
     window->priv->need_update_transient_for = TRUE;
+    window->priv->need_update_wmclass = TRUE;
+    window->priv->need_update_wmhints = TRUE;
     force_update_now (window);
 
     return window;
@@ -450,17 +465,14 @@ netk_window_get_xid (NetkWindow * window)
     return window->priv->xwindow;
 }
 
-/**
- * netk_window_get_session_id:
- * @window: a #NetkWindow
- *
- * Gets the session ID for @window in Latin-1 encoding.
- * NOTE: this is invalid UTF-8. You can't display this
- * string in a GTK widget without converting to UTF-8.
- * See netk_window_get_session_id_utf8().
- *
- * Return value: session ID in Latin-1
- **/
+NetkClassGroup *
+netk_window_get_class_group (NetkWindow *window)
+{
+  g_return_val_if_fail (NETK_IS_WINDOW (window), NULL);
+
+  return window->priv->class_group;
+}
+
 const char *
 netk_window_get_session_id (NetkWindow * window)
 {
@@ -527,6 +539,22 @@ netk_window_get_window_type (NetkWindow * window)
     return window->priv->wintype;
 }
 
+const char*
+netk_window_get_resource_class (NetkWindow *window)
+{
+  g_return_val_if_fail (NETK_IS_WINDOW (window), NULL);
+
+  return window->priv->res_class;
+}
+
+const char*
+netk_window_get_resource_name (NetkWindow *window)
+{
+  g_return_val_if_fail (NETK_IS_WINDOW (window), NULL);
+
+  return window->priv->res_name;
+}
+
 /**
  * netk_window_is_minimized:
  * @window: a #NetkWindow
@@ -1014,10 +1042,14 @@ get_icons (NetkWindow * window)
         window->priv->need_emit_icon_changed = TRUE;
 
         if (window->priv->icon)
+        {
             g_object_unref (G_OBJECT (window->priv->icon));
+        }
 
         if (window->priv->mini_icon)
+        {
             g_object_unref (G_OBJECT (window->priv->mini_icon));
+        }
 
         window->priv->icon = icon;
         window->priv->mini_icon = mini_icon;
@@ -1034,10 +1066,9 @@ netk_window_get_icon (NetkWindow * window)
 
     get_icons (window);
     if (window->priv->need_emit_icon_changed)
-        queue_update (window);  /* not done in get_icons since we call that from
-                                 * the update
-                                 */
-
+    {
+        queue_update (window);
+    }
     return window->priv->icon;
 }
 
@@ -1048,10 +1079,9 @@ netk_window_get_mini_icon (NetkWindow * window)
 
     get_icons (window);
     if (window->priv->need_emit_icon_changed)
-        queue_update (window);  /* not done in get_icons since we call that from
-                                 * the update
-                                 */
-
+    {
+        queue_update (window);
+    }
     return window->priv->mini_icon;
 }
 
@@ -1248,6 +1278,20 @@ p_netk_window_set_application (NetkWindow * window, NetkApplication * app)
     window->priv->app = app;
 }
 
+void
+p_netk_window_set_class_group (NetkWindow     *window,
+                             NetkClassGroup *class_group)
+{
+    g_return_if_fail (NETK_IS_WINDOW (window));
+    g_return_if_fail (class_group == NULL || NETK_IS_CLASS_GROUP (class_group));
+
+    if (class_group)
+        g_object_ref (G_OBJECT (class_group));
+    if (window->priv->class_group)
+        g_object_unref (G_OBJECT (window->priv->class_group));
+    window->priv->class_group = class_group;
+}
+
 void
 p_netk_window_process_property_notify (NetkWindow * window, XEvent * xevent)
 {
@@ -1301,14 +1345,23 @@ p_netk_window_process_property_notify (NetkWindow * window, XEvent * xevent)
         window->priv->need_update_wintype = TRUE;
         queue_update (window);
     }
+    else if (xevent->xproperty.atom == XA_WM_CLASS)
+    {
+        window->priv->need_update_wmclass = TRUE;
+        queue_update (window);
+    }
     else if (xevent->xproperty.atom == p_netk_atom_get ("_NET_WM_ICON")
-             || xevent->xproperty.atom == p_netk_atom_get ("KWM_WIN_ICON")
-             || xevent->xproperty.atom == p_netk_atom_get ("WM_HINTS"))
+             || xevent->xproperty.atom == p_netk_atom_get ("KWM_WIN_ICON"))
     {
         p_netk_icon_cache_property_changed (window->priv->icon_cache,
                                             xevent->xproperty.atom);
         queue_update (window);
     }
+    else if (xevent->xproperty.atom == p_netk_atom_get ("WM_HINTS"))
+    {
+        window->priv->need_update_wmhints = TRUE;
+        queue_update (window);
+    }
 }
 
 void
@@ -1720,6 +1773,35 @@ update_transient_for (NetkWindow * window)
     }
 }
 
+static void
+update_wmclass (NetkWindow *window)
+{
+    if (!window->priv->need_update_wmclass)
+      return;
+
+    window->priv->need_update_wmclass = FALSE;
+
+    g_free (window->priv->res_class);
+    g_free (window->priv->res_name);
+
+    p_netk_get_wmclass (window->priv->xwindow,
+                        &window->priv->res_class,
+                        &window->priv->res_name);
+}
+
+static void
+update_wmhints (NetkWindow *window)
+{
+    gboolean retval;
+    
+    if (!window->priv->need_update_wmhints)
+      return;
+
+    window->priv->need_update_wmhints = FALSE;
+    retval = p_netk_read_hints(window->priv->xwindow);
+    window->priv->is_urgent = (retval ? 1 : 0);
+}
+
 static void
 force_update_now (NetkWindow * window)
 {
@@ -1769,6 +1851,8 @@ force_update_now (NetkWindow * window)
     old_state = COMPRESS_STATE (window);
     old_actions = window->priv->actions;
 
+    update_wmclass (window);
+    update_wmhints (window);
     update_transient_for (window);      /* wintype needs this to be first */
     update_wintype (window);
     update_wm_state (window);
