@@ -31,13 +31,17 @@
 #include <string.h>
 #endif
 
-#include <libxfce4util/utf8.h>
+#include <libxfce4util/libxfce4util.h>
 
 #include "netk-xutils.h"
 #include "netk-screen.h"
 #include "netk-private.h"
 #include "netk-inlinepixbufs.h"
 
+#ifndef XUrgencyHint
+#define XUrgencyHint (1L << 8)
+#endif
+
 gboolean
 p_netk_get_cardinal (Window xwindow, Atom atom, int *val)
 {
@@ -255,7 +259,7 @@ p_netk_get_text_property (Window xwindow, Atom atom)
         retval = text_property_to_utf8 (&text);
         if (retval)
         {
-            utf8_string_remove_controls((gchar *) retval, -1, NULL);
+            xfce_utf8_remove_controls((gchar *) retval, -1, NULL);
         }
         if (text.nitems > 0)
             XFree (text.value);
@@ -345,7 +349,7 @@ p_netk_get_utf8_property (Window xwindow, Atom atom)
         return NULL;
     }
 
-    utf8_string_remove_controls((gchar *) val, nitems, NULL);
+    xfce_utf8_remove_controls((gchar *) val, nitems, NULL);
     retval = g_strndup (val, nitems);
 
     XFree (val);
@@ -540,7 +544,7 @@ p_netk_get_utf8_list (Window xwindow, Atom atom)
             return NULL;
         }
 
-        utf8_string_remove_controls((gchar *) p, -1, NULL);
+        xfce_utf8_remove_controls((gchar *) p, -1, NULL);
         retval[i] = g_strdup (p);
 
         p = p + strlen (p) + 1;
@@ -1042,12 +1046,24 @@ latin1_to_utf8 (const char *latin1)
     return g_string_free (str, FALSE);
 }
 
-char *
+char*
 p_netk_get_res_class_utf8 (Window xwindow)
 {
+
+    char *res_class;
+
+    p_netk_get_wmclass (xwindow, &res_class, NULL);
+
+    return res_class;
+}
+
+void
+p_netk_get_wmclass (Window xwindow, char **res_class, char **res_name)
+{
+
     XClassHint ch;
     char *retval;
-
+  
     p_netk_error_trap_push ();
 
     ch.res_name = NULL;
@@ -1056,21 +1072,38 @@ p_netk_get_res_class_utf8 (Window xwindow)
     XGetClassHint (gdk_display, xwindow, &ch);
 
     p_netk_error_trap_pop ();
-
+  
     retval = NULL;
 
+
+    if (res_class)
+    {
+        *res_class = NULL;
+    }
+
+    if (res_name)
+    {
+        *res_name = NULL;
+    }
+
+  
     if (ch.res_name)
     {
+        if (res_name)
+        {
+            *res_name = latin1_to_utf8 (ch.res_name);
+        }
         XFree (ch.res_name);
     }
 
     if (ch.res_class)
     {
-        retval = latin1_to_utf8 (ch.res_class);
+        if (res_class)
+        {
+            *res_class = latin1_to_utf8 (ch.res_class);
+        }
         XFree (ch.res_class);
     }
-
-    return retval;
 }
 
 void
@@ -1429,8 +1462,10 @@ get_cmap (GdkPixmap * pixmap)
     if (cmap
         && (gdk_colormap_get_visual (cmap)->depth !=
             gdk_drawable_get_depth (pixmap)))
+    {
+        g_object_unref (G_OBJECT (cmap));
         cmap = NULL;
-
+    }
     return cmap;
 }
 
@@ -1789,6 +1824,26 @@ scaled_from_pixdata (guchar * pixdata, int w, int h, int new_w, int new_h)
     return dest;
 }
 
+gboolean
+p_netk_read_hints (Window xwindow)
+{
+    XWMHints *hints;
+    gboolean retval;
+    retval = FALSE;
+    
+    p_netk_error_trap_push ();
+    hints = XGetWMHints (gdk_display, xwindow);
+    p_netk_error_trap_pop ();
+    
+    if (hints)
+    {
+        retval  = (hints->flags & XUrgencyHint);
+        XFree (hints);
+        hints = NULL;
+    }
+    return retval;
+}
+  
 gboolean
 p_netk_read_icons (Window xwindow, NetkIconCache * icon_cache,
                    GdkPixbuf ** iconp, int ideal_width, int ideal_height,
