@@ -1,4 +1,4 @@
-/* $Id: xfce_iconbutton.c,v 1.12.2.6 2004/06/20 19:46:04 jasper Exp $
+/* $Id: xfce_iconbutton.c,v 1.20 2004/07/16 17:37:36 kelnos Exp $
  * 
  * Copyright 2002-2004 Jasper Huijsmans (jasper@xfce.org)
  *
@@ -44,6 +44,20 @@ static void xfce_iconbutton_init (XfceIconbutton * iconbutton);
 
 static GtkWidgetClass *parent_class = NULL;
 
+/* workaround for http://bugzilla.gnome.org/show_bug.cgi?id=142417 */
+static void
+_style_set_cb(GtkWidget *w, GtkStyle *prev_style, gpointer user_data)
+{
+    GtkStyle *style;
+
+    if (XFCE_ICONBUTTON (w)->image)
+    {
+        style = gtk_rc_get_style_by_paths (gtk_settings_get_default (),
+            "GtkButton", "GtkButton", GTK_TYPE_BUTTON);
+        gtk_widget_set_style (XFCE_ICONBUTTON (w)->image, style);
+    }
+}
+
 GtkType
 xfce_iconbutton_get_type (void)
 {
@@ -91,7 +105,12 @@ xfce_iconbutton_init (XfceIconbutton * iconbutton)
 GtkWidget *
 xfce_iconbutton_new (void)
 {
-    return GTK_WIDGET (g_object_new (xfce_iconbutton_get_type (), NULL));
+    GtkWidget *iconbutton = GTK_WIDGET (g_object_new (xfce_iconbutton_get_type (), NULL));
+
+    g_signal_connect (G_OBJECT (iconbutton), "style-set",
+        G_CALLBACK (_style_set_cb), NULL);
+
+    return iconbutton;
 }
 
 GtkWidget *
@@ -102,6 +121,9 @@ xfce_iconbutton_new_from_pixbuf (GdkPixbuf * pixbuf)
     iconbutton = xfce_iconbutton_new ();
     xfce_iconbutton_set_pixbuf (XFCE_ICONBUTTON (iconbutton), pixbuf);
 
+    g_signal_connect (G_OBJECT (iconbutton), "style-set",
+        G_CALLBACK (_style_set_cb), NULL);
+
     return iconbutton;
 }
 
@@ -114,5 +136,3 @@ xfce_iconbutton_set_pixbuf (XfceIconbutton * iconbutton, GdkPixbuf * pixbuf)
     xfce_scaled_image_set_from_pixbuf (XFCE_SCALED_IMAGE (iconbutton->image),
 	    			       pixbuf);
 }
-
-
