@@ -1,4 +1,4 @@
-/* $Id: xfce_scaled_image.c,v 1.1 2004/06/15 18:21:58 jasper Exp $
+/* $Id: xfce_scaled_image.c,v 1.3 2004/08/22 22:55:10 dannym Exp $
  * 
  * Copyright 2004 Jasper Huijsmans (jasper@xfce.org)
  *
@@ -132,8 +132,11 @@ xfce_scaled_image_set_from_pixbuf (XfceScaledImage * image, GdkPixbuf * pixbuf)
     if (image->pb)
 	g_object_unref (image->pb);
 
+
     image->pb = pixbuf;
-    g_object_ref (image->pb);
+    
+    if (image->pb) 
+        g_object_ref (image->pb);
 
     if (image->width > 0)
     {
@@ -204,6 +207,9 @@ xfce_scaled_image_scale (XfceScaledImage * image)
     double wratio, hratio;
     int w, h, pw, ph;
 
+    if (!image->pb)
+       return FALSE;
+       
     g_return_val_if_fail (GDK_IS_PIXBUF (image->pb), FALSE);
 
     pw = gdk_pixbuf_get_width (image->pb);
